# -*- coding: utf-8 -*-
from pygooglechart import *

import re
import sys
import signal
import time
import sqlite3 as sqlite
import math

class BasePlugin(object):

    __connector = None

    def _get_database_path(self):
        return os.path.expanduser('~/.hwm.db')

    def get_connector(self):
        return sqlite.connect(self._get_database_path())

    def __init__(self, params):
        self._params = params
        self._die = False
        self._period = 300.0
    
    @classmethod
    def description(class_):
        return {
            'name': None,
            'slug': None,
            'options': []
        }

class BaseColectorPlugin(BasePlugin):

    def clear(self):
        raise NotImplementedError('Method abstract')

    def _prepare(self, **kargs):
        pass

    def get_data_sequence(self, **kargs):
        return self._get_data_sequencce(**kargs)

    def get_column(self):
        return tuple()

    def reduce_number(self, number):
        if number == 0:
            return 0, 1
        else:
            return float(number) / (float(10) ** int(math.log(number, 10))), math.log(number, 10)

    def calc_limits(self, start = None, end = None):
        if end is None: raise Exception('Deve especificar o final do range')
        if start is None: start = 0

        base, exp = self.reduce_number(end)
        return start, int((int(base) + 1) * (10 ** int(exp)))

    def _get_data_sequencce(self, **kargs):
        seq = [seq for seq in self.do_extract(**kargs)]
        return {
            'column': self.get_column(),
            'collection': tuple(seq),
            'count': len(seq)
        }

    def _create_table(self):
        pass
    
    def do_colect(self):
        raise NotImplementedError()
    
    def do_extract(self, **kargs):
        raise NotImplementedError()

    def handle_sigint(self, signal, frame):
        self._die = True
        print ''
        print 'Processo finalizado'
        print ''

    def colect(self):
        self.do_colect()

    def extract(self):
        params = self._params.__dict__
        self.do_extract(**params)

class BaseGraphicPlugin(BasePlugin):

    _default_image_size = (550,250)

    def _config_markers(self, chart, index, color, fill = True):
        if fill is True:
            chart.add_marker(index, 0, 'B', '%s90' % color, 2)
        else:
            chart.add_marker(index, 0, 'B', '%s40' % color, 2)

        chart.add_marker(index, -1, 'o', color, 5)
        chart.add_marker(index, -1, 'o', 'ffffff', 3)
    
    def get_image_object(self):
        return Image.new(mode = 'RGB', size = self._default_image_size)

    def humanize(self, number):
        return number

    def draw(self):
        data = BaseColectorPlugin.get_data_sequence(self, **self.params)
        filename = self.filename

        max_y = 0
        for item in data.get('collection')[1:]:
            for value in item: max_y = value if value > max_y else max_y

        min_y, max_y = self.calc_limits(end = max_y)
        space_y = max_y - min_y
        part_y = (space_y / 10)

        slot_count = len(data.get('collection')[0])

        chart = SimpleLineChart(650, 150, y_range = (min_y, max_y), colours = self.colors)
        for slot in xrange(slot_count):
            chart.add_data([item[slot] for item in data.get('collection')[1:]])
            self._config_markers(chart, slot, self.colors[slot])
            
        chart.set_axis_labels(Axis.LEFT, ['%0.1f %s' % self.humanize(x) for x in xrange(min_y, max_y, part_y)])
        chart.set_axis_labels(Axis.RIGHT, ['%0.1f %s' % self.humanize(x) for x in xrange(min_y, max_y, part_y)])
        chart.set_grid(
            50,
            11.15,
            1,
            2
        )

        chart.download(filename)

class PluginManager:

    class PluginNotFound(Exception):

        def __init__(self, slug):
            Exception.__init__(self, u'The slug %s not found in base of plugins.' % slug)

    class BasePluginNotCompatible(Exception):

        def __init__(self):
            Exception.__init__(self, u'The base not is an BasePlugin valid.')


    def __init__(self, base):
        if issubclass(base, BasePlugin) is True:
            self._database = {}
            self._basePlugin = base
            self._scan()
        else:
            raise self.BasePluginNotCompatible()

    def _scan(self):
        pdir = os.path.dirname(os.path.abspath(__file__))
        to_match = '^(?P<module>.+)\.py$'
        files = [filename for filename in os.listdir(pdir) if re.match(to_match, filename)]
        modules = [re.match(to_match, filename).groupdict().get('module') for filename in files if filename not in ['__init__.py', 'base.py']]

        for module in modules:
            sys.stdout.write('Loading module %s ... ' % module)
            loaded = None
            try:
                exec('from hwm.plugin import %s as loaded' % module)
            except:
                print 'fail'
            else:
                in_module = []
                for attr in dir(loaded):
                    flag = False
                    plg = getattr(loaded, attr)
                    try:
                        flag = (attr != 'BaseColectorPlugin' and issubclass(plg, self._basePlugin))
                    except:
                        pass
                    else:
                        if flag is True:
                            self._database.update({plg.description().get('slug'): plg})
                            in_module.append(str(plg.description().get('name')))
                sys.stdout.write('(')
                sys.stdout.write(', '.join(in_module))
                sys.stdout.write(')')

                print ' ok'

    def get(self, slug):
        if slug in self._database.keys():
            return self._database.get(slug)
        else:
            raise self.PluginNotFound(slug)